=begin
------------------------------------------------------------------------
Copyright 2008, TIG (c)
Permission to use, copy, modify, and distribute this software for any 
purpose, and currently without fee, is hereby granted, provided that  
this text and the above copyright (c) notice appear in all copies.
This software is provided "as is" and without any express or implied 
warranties, including, without limitation, the implied warranties of 
merchantability and fitness for a particular purpose.
------------------------------------------------------------------------

Name        : TextTag.rb

Author      : TIG

Description : A tool that adds a 'flat' 3D text-tag component, 
             that's placed 0.1mm above any face so it remains visible.
			 
			You are first prompted for the 'text'...
			  Note: type \n for any new-line breaks you require, 
			  e.g. "Text\nTag on\n3 Lines".
			You can choose to change settings, Yes/No: No = skip,
			  the initial def=Yes: it accesses the following settings.
			   height = of upper-case I in current units: def=100.mm/4".
			   alignment = Centred, Left, Right: default=Centred.
			   font = Most TTF Fonts are listed: default=Comic Sans MS
				[See 'FONTLIST...' below to edit list or add your own...
				the list is based on most MS Vista TTFs so Mac users 
				might like to add "Helvetica" etc]
			    [if lists off screen, type start letter, use arrow keys...]
			   bold = Yes/No: default=No.
			   italic = Yes/No: default=No.
			   filled = Yes/No: default=Yes, No=Outline - Edges only.
			   color = Any Existing/Standard Color: default=Black.
			    using <Default> lets instances be painted other colors.
			    [if lists off screen, type start letter, use arrow keys...]
			   layer = Default=TEXT-TAGS, pick another OR make your own.
			    [if lists off screen, type start letter, use arrow keys...]
			   face-camera = Yes/No: default=No -  best for most uses.
			   snap = Yes/No: default=Yes - then tag snaps onto picked 
			   face - you can always 'unglue' snapped tags later.
			These settings are remembered in the model and as attributes 
			in the individual tag definitions.
			The Tool repeats, Cancel/Esc to Exit.
			Each tag has a unique name, e.g. "~TextTag#7", 
			  so they will list at the end of the model's components.
			Any text-tag is editable by right-click context-menu...
			  This lets you remake the tag's values as above. except that 
			  the tag remains where it was originally placed down.
			  Tags can be moved and copied, BUT remember that changes to 
			  a copy will affect its siblings, if it's not 'made-unique'.
			The tag's letters, spacing etc can be edited within the tag 
			instance, BUT remember changes will be lost after editing...
			
			To repeat TextTag Tool make a shortcut key - say Shift+T ?
			
Menu Item   : Plugins > 'TextTag'

Context Menu: 'Edit TextTag' [if a text tag is selected]

Version     : 1.0	20071209	First issue.
			  1.1	20071211	Text-box now bigger: 64 trailing spaces are
								missed in tag text.
			  1.2	20071222	Dialog allows 'stand-off' of tag to be set,
								default = 0.1" or 1mm.
								Cast/Receive Shadows can also now be set.
			  1.3	20071231	Glitch on Edit TextTag selection fixed.
								Proto-DXF exporter continues...
        1.4 20080313  Filled option now 'Yes', 'Yes+Edges' and 'No'
                - useful for exports to dewg etc as hidden edges not seen...
								DXF exporter continues - see v1.5...
								
=end
require 'sketchup.rb'
###
class TextTag
### FONTLIST...
### ADD any extra fonts YOU normally use [or delete those don't ever want to use]...
@fontlist=["Arial","Arial Black","Arial Narrow","Bookman Old Style","Bradley Hand ITC","Century","Century Gothic","CityBlueprint","Corbel","Comic Sans MS","Courier New","Eurostile","Garamond","Georgia","Impact","Lucida Console","Monotxt","Monotype Corsiva","Papyrus","Rockwell","RomanT","Romantic","SansSerif","Simplex","Stylus BT","Symbol","Tahoma","Technic","Times New Roman","Trebuchet MS","Verdana","Vineta BT","Vrinda","Webdings","Wingdings","Wingdings 2","Wingdings 3"]###end of font list
###
def TextTag::new
model=Sketchup.active_model
entities=model.active_entities
definitions=model.definitions
model.start_operation("TextTag")
### 3D TEXT TAG Settings
### see if model is tagged
ht=4.inch if model.options["UnitsOptions"]["LengthUnit"]<2
ht=100.mm if model.options["UnitsOptions"]["LengthUnit"]>1
so=0.1.inch if model.options["UnitsOptions"]["LengthUnit"]<2###
so=1.mm if model.options["UnitsOptions"]["LengthUnit"]>1###
@height=model.get_attribute("TextTag","height",ht)
@align=model.get_attribute("TextTag","align","Center")
@font=model.get_attribute("TextTag","font","Comic Sans MS")
@bold=model.get_attribute("TextTag","bold","No")
@italic=model.get_attribute("TextTag","italic","No")
@filled=model.get_attribute("TextTag","filled","Yes")
@color=model.get_attribute("TextTag","color","Black")
@layername=model.get_attribute("TextTag","layername","TEXT-TAGS")
@cfacing=model.get_attribute("TextTag","cfacing","No")
@asnap=model.get_attribute("TextTag","asnap","Yes")
@standoff=model.get_attribute("TextTag","standoff",so)###
@shadows=model.get_attribute("TextTag","shadows","No-Cast+No-Receive")###
### NOW set text and other
@text="TextTag                                                  " if not @text
### +64 trailing spaces
@other="Yes" if not @other ### to access other settings
###
### some other settings that aren't editable
@tol=1.mm ### or other tolerance (or units e.g. 0.1 = inches)
@z=0 ### base 'z' value
@extn=0 ### extrusion in " (or units e.g. 10.mm), only used if 'filled'...
### dialog
### show VCB and status info
  Sketchup::set_status_text(("TextTag Settings..." ), SB_PROMPT)
  Sketchup::set_status_text(" ", SB_VCB_LABEL)
  Sketchup::set_status_text(" ", SB_VCB_VALUE)
  return nil if not TextTag.dialog ### do dialog...
  Sketchup::set_status_text(("TextTag Place..." ), SB_PROMPT)
### sort results
### ensure \n becomes a line break in text
### AND get rid of any trailing spaces...
  @text=@text.strip+"                                                  "### 64 spaces
  text=@text.strip.gsub(/\\n/,"\n")### removes trailing/leading spaces
### align 0/1/2
  align=0 if @align=="Left"
  align=1 if @align=="Center"
  align=2 if @align=="Right"
### other settings Y/N = True/False
  bold=true;bold=false if @bold=="No"
  italic=true;italic=false if @italic=="No"
  filled=@filled
### sort layer
layers=model.layers
layer=false
layers.each{|e|layer=e if e.name==@layername}
if not layer
  layer=layers.add(@layername)
  pb=layer.page_behavior=LAYER_IS_HIDDEN_ON_NEW_PAGES
  model.pages.each{|p|p.set_visibility(layer,false)}
  model.pages.selected_page.set_visibility(layer,true)
end#if
### make component
compo=definitions.add("~TextTag")
definitions.unique_name("~TextTag")
compo.description="Text Tag Object"
ents=compo.entities
### lift compo up @standoff so can be seen above faces
pt=Geom::Point3d.new(0,0,@standoff)
t=Geom::Transformation.new(pt)
g=ents.add_group(); gents=g.entities
gents.add_3d_text(text,align,@font,bold,italic,@height,@tol,@z,filled,@extn)
g.move!(t); g.explode
### sort out edges and faces of text
ents.each{|e|
  e.layer=layer
  e.hidden=true if e.typename=="Edge" and filled=="Yes"
  e.hidden=true if e.typename=="Edge" and filled=="Yes"
  e.hidden=true if e.typename=="Edge" and filled=="Yes"
  ### repeats to catch glitch
  if e.typename=="Face"
    e.material=@color
    e.back_material=@color
	  e.reverse! ### so faces up
    e.erase! if filled=="No"
  end#if
}
### refix colors
@color="<Default>" if @color==nil
### refix layers
@layername="<Default>" if @layername==nil
compo.behavior.always_face_camera=true if @cfacing=="Yes" ### faces camera
if @asnap=="Yes"
  compo.behavior.is2d=true
  compo.behavior.snapto=0  ### snaps to Any surface
end#if
if @shadows=~/No-Cast/###
  ents.each{|e|e.casts_shadows=false}
else
  ents.each{|e|e.casts_shadows=true}
end#if###
if @shadows=~/No-Receive/### 
  ents.each{|e|e.receives_shadows=false}
else
  ents.each{|e|e.receives_shadows=true}
end#if###
### set tag compo attributes
compo.set_attribute("TextTag","text",@text)
compo.set_attribute("TextTag","height",@height)
compo.set_attribute("TextTag","align",@align)
compo.set_attribute("TextTag","font",@font)
compo.set_attribute("TextTag","bold",@bold)
compo.set_attribute("TextTag","italic",@italic)
compo.set_attribute("TextTag","filled",@filled)
compo.set_attribute("TextTag","color",@color)
compo.set_attribute("TextTag","layername",@layername)
compo.set_attribute("TextTag","cfacing",@cfacing)
compo.set_attribute("TextTag","asnap",@asnap)
### give model current settings as attributes
model.set_attribute("TextTag","height",@height)
model.set_attribute("TextTag","align",@align)
model.set_attribute("TextTag","font",@font)
model.set_attribute("TextTag","bold",@bold)
model.set_attribute("TextTag","italic",@italic)
model.set_attribute("TextTag","filled",@filled)
model.set_attribute("TextTag","color",@color)
model.set_attribute("TextTag","layername",@layername)
model.set_attribute("TextTag","cfacing",@cfacing)
model.set_attribute("TextTag","asnap",@asnap)
model.get_attribute("TextTag","standoff",@standoff)###
model.get_attribute("TextTag","shadows",@shadows)###
### place tag
tag=model.place_component(compo)
### end undo
model.commit_operation
###
end#def TextTag::new
###
def TextTag::dialog
  model=Sketchup.active_model
  ### get text and if other settings
  prompts=["Enter Text: ","Edit Settings:"]
  values=[@text,@other]
  popups=["","Yes|No"]
  title="TextTag"
  results=inputbox(prompts,values,popups,title)
  return nil if not results
  @text,@other=results
  ### do 'other' dialog
  if @other=="Yes"
	aligns=["Left|Center|Right"]
	fonts=@fontlist
	fonts.sort!; fonts=fonts.join("|")
	### colors
    materials=model.materials
	colors=[]; materials.each{|m|colors.push(m.display_name)}
	colors=colors+["<Default>"]+(Sketchup::Color.names.sort!)
	colors.uniq; colors=colors.join("|")
	### layers
    mlayers=model.layers; layers=[]
	mlayers.each{|e|layers.push(e.name)}
    dlayer=layers[0]; layers=layers-[dlayer]; layers.sort!
    ### sort for possible new layer...
    makelayer="<Make New Layer>"
    layers=[makelayer]+[dlayer]+["TEXT-TAGS"]+layers; 
	layers.uniq!; layers=layers.join('|')
	### dialog...
    prompts=["Height: ","Alignment:","Font: ","Bold: ","Italic: ","Filled: ","Color: ","Layer: ","Face-Camera: ","Snap-to-Surface: ","Stand-off: ","Shadow Settings: "]
    values=[@height,@align,@font,@bold,@italic,@filled,@color,@layername,@cfacing,@asnap,@standoff,@shadows]
    popups=["",aligns,fonts,"Yes|No","Yes|No","Yes|Yes+Edges|No",colors,layers,"Yes|No","Yes|No","","No-Cast+No-Receive|No-Cast+Receive|Cast+No-Receive|Cast+Receive"]
    title="TextTag Settings"
    results=inputbox(prompts,values,popups,title)
    return nil if not results
    @height,@align,@font,@bold,@italic,@filled,@color,@layername,@cfacing,@asnap,@standoff,@shadows=results
	### sort out colors
	@color=nil if @color=="<Default>"
	### sort out layers
	@layername=nil if @layername=="<Default>"
	if @layername=="<Make New Layer>"### another dialog...
	  results=inputbox(["New Layer Name: "],["????"],"TextTag: New Layer")
      if results
        @layername=results[0]
      else
        @layername=nil
      end#if
	end#if
  end#if
  return true
end#def TextTag::dialog
###
###
def TextTag::edit
model=Sketchup.active_model
entities=model.active_entities
stag=model.selection[0]
stag=stag.definition
definitions=model.definitions
model.start_operation("Edit TextTag")
### get def's tags
ht=4.inch if model.options["UnitsOptions"]["LengthUnit"]<2
ht=100.mm if model.options["UnitsOptions"]["LengthUnit"]>1
so=0.1.inch if model.options["UnitsOptions"]["LengthUnit"]<2###
so=1.mm if model.options["UnitsOptions"]["LengthUnit"]>1###
@sheight=stag.get_attribute("TextTag","height",ht)
@salign=stag.get_attribute("TextTag","align","Center")
@sfont=stag.get_attribute("TextTag","font","Comic Sans MS")
@sbold=stag.get_attribute("TextTag","bold","No")
@sitalic=stag.get_attribute("TextTag","italic","No")
@sfilled=stag.get_attribute("TextTag","filled","Yes")
@scolor=stag.get_attribute("TextTag","color","Black")
@slayername=stag.get_attribute("TextTag","layername","TEXT-TAGS")
@scfacing=stag.get_attribute("TextTag","cfacing","No")
@sasnap=stag.get_attribute("TextTag","asnap","Yes")
@sstandoff=stag.get_attribute("TextTag","standoff",so)###
@sshadows=stag.get_attribute("TextTag","shadows","No-Cast+No-Receive")###
### NOW get text and other
@stext=stag.get_attribute("TextTag","text","????")
@sother="Yes" if not @sother ### to access other settings
###
### some other settings that aren't editable
@stol=1.mm ### or other tolerance (or units e.g. 0.1 = inches)
@sz=0 ### base 'z' value
@sextn=0 ### extrusion in " (or units e.g. 10.mm), only used if 'filled'...
### dialog
### show VCB and status info
  Sketchup::set_status_text(("Edit TextTag Settings..." ), SB_PROMPT)
  Sketchup::set_status_text(" ", SB_VCB_LABEL)
  Sketchup::set_status_text(" ", SB_VCB_VALUE)
  return nil if not TextTag.sdialog ### do dialog...
### sort results
### ensure \n becomes a line break in text
  @stext=@stext.strip+"                                                  "
  ### 64 spaces
  text=@stext.strip.gsub(/\\n/,"\n")### removes trailing/leading spaces
### align 0/1/2
  align=0 if @salign=="Left"
  align=1 if @salign=="Center"
  align=2 if @salign=="Right"
### other settings Y/N = True/False
  bold=true;bold=false if @sbold=="No"
  italic=true;italic=false if @sitalic=="No"
  filled=@sfilled
### sort layer
layers=model.layers
layer=false
layers.each{|e|layer=e if e.name==@slayername}
if not layer
  layer=layers.add(@slayername)
  pb=layer.page_behavior=LAYER_IS_HIDDEN_ON_NEW_PAGES
  model.pages.each{|p|p.set_visibility(layer,false)}
  model.pages.selected_page.set_visibility(layer,true)
end#if
### get component
compo=stag ### compo is stag = the selected intance's definintion
ents=compo.entities
xents=ents.to_a ###***
xents.each{|e|e.erase! if e.valid?}### remove old stuff***
### lift compo up @standoff so can be seen above faces
pt=Geom::Point3d.new(0,0,@sstandoff)
t=Geom::Transformation.new(pt)
g=ents.add_group(); gents=g.entities
gents.add_3d_text(text,align,@sfont,bold,italic,@sheight,@stol,@sz,filled,@sextn)
g.move!(t); g.explode
### sort out edges and faces of text
ents.each{|e|
  e.layer=layer
  e.hidden=true if e.typename=="Edge" and filled=="Yes"
  e.hidden=true if e.typename=="Edge" and filled=="Yes"
  e.hidden=true if e.typename=="Edge" and filled=="Yes"
  ### repeats to catch glitch
  if e.typename=="Face"
    e.material=@scolor
    e.back_material=@scolor
	  e.reverse! ### so faces up
    e.erase! if filled=="No"
  end#if
}
### refix colors
@scolor="<Default>" if @scolor==nil
### refix layers
@slayername="<Default>" if @slayername==nil
compo.behavior.always_face_camera=true if @scfacing=="Yes" ### faces camera
if @sasnap=="Yes"
  compo.behavior.is2d=true
  compo.behavior.snapto=0  ### snaps to Any surface
end#if
if @sshadows=~/No-Cast/###
  ents.each{|e|e.casts_shadows=false}
else
  ents.each{|e|e.casts_shadows=true}
end#if###
if @sshadows=~/No-Receive/### 
  ents.each{|e|e.receives_shadows=false}
else
  ents.each{|e|e.receives_shadows=true}
end#if###
### set tag compo attributes
compo.set_attribute("TextTag","text",@stext)
compo.set_attribute("TextTag","height",@sheight)
compo.set_attribute("TextTag","align",@salign)
compo.set_attribute("TextTag","font",@sfont)
compo.set_attribute("TextTag","bold",@sbold)
compo.set_attribute("TextTag","italic",@sitalic)
compo.set_attribute("TextTag","filled",@sfilled)
compo.set_attribute("TextTag","color",@scolor)
compo.set_attribute("TextTag","layername",@slayername)
compo.set_attribute("TextTag","cfacing",@scfacing)
compo.set_attribute("TextTag","asnap",@sasnap)
compo.set_attribute("TextTag","standoff",@sstandoff)###
compo.set_attribute("TextTag","shadows",@sshadows)###
### end undo
model.commit_operation
###
end#def TextTag::edit
###
def TextTag::sdialog
  model=Sketchup.active_model
  ### get text and if other settings
  prompts=["Enter Text: ","Edit Settings:"]
  values=[@stext,@sother]
  popups=["","Yes|No"]
  title="TextTag"
  results=inputbox(prompts,values,popups,title)
  return nil if not results
  @stext,@sother=results
  ### do 'other' dialog
  if @sother=="Yes"
	aligns=["Left|Center|Right"]
	fonts=@fontlist
	fonts.sort!; fonts=fonts.join("|")
	### colors
    materials=model.materials
	colors=[]; materials.each{|m|colors.push(m.display_name)}
	colors=colors+["<Default>"]+(Sketchup::Color.names.sort!)
	colors.uniq; colors=colors.join("|")
	### layers
    mlayers=model.layers; layers=[]
	mlayers.each{|e|layers.push(e.name)}
    dlayer=layers[0]; layers=layers-[dlayer]; layers.sort!
    ### sort for possible new layer...
    makelayer="<Make New Layer>"
    layers=[makelayer]+[dlayer]+["TEXT-TAGS"]+layers; 
	layers.uniq!; layers=layers.join('|')
	### dialog...
    prompts=["Height: ","Alignment:","Font: ","Bold: ","Italic: ","Filled: ","Color: ","Layer: ","Face-Camera: ","Snap-to-Surface: ","Stand-off: ","Shadow Settings: "]
    values=[@sheight,@salign,@sfont,@sbold,@sitalic,@sfilled,@scolor,@slayername,@scfacing,@sasnap,@sstandoff,@sshadows]
    popups=["",aligns,fonts,"Yes|No","Yes|No","Yes|Yes+Edges|No",colors,layers,"Yes|No","Yes|No","","No-Cast+No-Receive|No-Cast+Receive|Cast+No-Receive|Cast+Receive"]
    title="TextTag Settings"
    results=inputbox(prompts,values,popups,title)
    return nil if not results
    @sheight,@salign,@sfont,@sbold,@sitalic,@sfilled,@scolor,@slayername,@scfacing,@sasnap,@sstandoff,@sshadows=results
    ### sort out colors
	@scolor=nil if @scolor=="<Default>"
	### sort out layers
	@slayername=nil if @slayername=="<Default>"
	if @slayername=="<Make New Layer>"### another dialog...
	  results=inputbox(["New Layer Name: "],["????"],"TextTag: New Layer")
      if results
        @slayername=results[0]
      else
        @slayername=nil
      end#if
	end#if
  end#if
  return true
end#def TextTag::dialog
###

###
def TextTag::dxf
  ### dxf exporter
  UI.messagebox("TextTags to DXF Exporter coming soon....")
end#def TextTag::dxf
###

###
def TextTag::ok
  s=Sketchup.active_model.selection[0]
  t=s.definition.get_attribute("TextTag","text",false)###v1.3fix
  if t
    return true
  else
    return false
  end#if
end#def TextTag::ok
###
end#class
###
#--------- menu -----------------------------
is_me=__FILE__
if not file_loaded?(is_me)
   ###add_separator_to_menu("Plugins")
   UI.menu("Plugins").add_item("TextTag"){TextTag.new}
   ###UI.menu("Plugins").add_item("TextTags to DXF"){TextTag.dxf}
   UI.add_context_menu_handler do |menu|
     ss=Sketchup.active_model.selection
      if (not ss[1] and ss[0].typename=="ComponentInstance") and TextTag.ok
         menu.add_separator
         menu.add_item("Edit TextTag"){TextTag.edit}
      end #if ok
   end #do menu
end#if
file_loaded(is_me)
#---------------------------------
